#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Module for Udev Event Handler."""


import fal_info as info

import util


class UdevHandler:
    """Class for the common udev event handler."""

    def _do_cancel(self):
        """Return if the event handling should be canceled."""

        if util.get_system_state() in util.CANCEL_STATE_LIST:
            return True

        return False

    def _try_udev_info_lock_or_unlock(self): 
        """Try locking or unlocking a udev info."""

        return util.try_udev_info_lock_or_unlock(self.device_name)

    def _get_event_action(self): 
        """Return the event action for the current udev event.
           None is returned if no action is necessary.
        """

        if util.is_action_add():
            return util.EVENT_ACTION_PLUG
        elif util.is_action_change():
            return util.EVENT_ACTION_PLUG
        elif util.is_action_remove():
            return util.EVENT_ACTION_UNPLUG

        # Do nothing
        return None

    def _update_info(self):
        """Update the device information and so on if necessary.
           If this method return non-zero value then handler return soon.
        """

        # By default, do nothing
        return 0

    def _try_log_eventlog(self): 
        """Try log to eventlog for a device hotplug event."""

        event_action = self._get_event_action()
        (port, port_no) = util.dev_to_port_info(self.device_name)
        if (event_action in self.eventlog_code_map and
            port_no in self.eventlog_code_map[event_action]):

            util.log_eventlog(self.eventlog_code_map[event_action][port_no])

    def _dispatch_event_and_buzzer(self): 
        """Dispach a device hotplug event and show result by buzzer."""

        event_action = self._get_event_action()
        body = util.get_device_event_body(self.device_name)
        result_entry = util.dispatch_event(self.event_category,
                                            event_action, body)

        if result_entry.get_result() == info.Results.OK:
            util.set_buzzer(info.BUZZER_OK)
        else:
            util.set_buzzer(info.BUZZER_ERROR)

    def __init__(self, event_category, eventlog_code_map):
        """Constructor."""

        self.event_category = event_category
        self.eventlog_code_map = eventlog_code_map
        self.device_name = util.get_device_name()

    def handle_udev_event(self):
        """Handle a udev event."""

        if self._do_cancel():
            return

        if self._try_udev_info_lock_or_unlock(): 
            return

        if self._get_event_action() == None:
            return

        util.try_background_exec()

        if self._update_info():
            return

        util.set_status_led(util.LED_BLINK)
        util.wait_udev_settle()
        util.wait_system_ready()

        self._try_log_eventlog()
        self._dispatch_event_and_buzzer()

        util.set_status_led(util.LED_ON)
